--[[

This is a simple quickslot window that
stores the quickslot in a rolodex type
manner.  Sorta like the quickslots in
DDO.

]]--

import "Turbine.Gameplay";
import "Turbine.UI.Extensions";
import "Turbine.Debug"

pagedSlotsWindow = class( Turbine.UI.Extensions.SimpleWindow );

function pagedSlotsWindow:Constructor()
	Turbine.UI.Extensions.SimpleWindow.Constructor( self );

	-- load the last settings saved
	self:LoadSettings();

	--[[ Here are the global variables ]]--
	self.pageNumber = 1;
	self.draggingShortcut = false;
	self.draggedOut = false;
	self.swappingShortcuts = false;
	self.ReplacedShortcut = nil;
	self.loading = false;
	self.bgColor = Turbine.UI.Color(0.3,0.5,0.5,0.5)

	--[[ set window properties ]]--
	self:SetSize( 250, 46 );
	self:SetBackColor(Turbine.UI.Color(0,0,0,0));
	self:SetPosition(self.settings.positionX,self.settings.positionY);
	self:SetOpacity( 1 );
	self:SetZOrder(1);
	self:SetAllowDrop( true );
	self:SetVisible(true);
	
	--[[ Add the label to the window ]]--
	self.pageLabel = Turbine.UI.Label();
	self.pageLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);

	self.pageLabel:SetBackColor(Turbine.UI.Color(0,0,0,0));
	self.pageLabel:SetForeColor(Turbine.UI.Color(1,1,1,1));
	self.pageLabel:SetBackColorBlendMode(Turbine.UI.BlendMode.Undefined);
	self.pageLabel:SetZOrder(99);
	self.pageLabel:SetParent(self);

	--[[ Add the page up and down buttons ]]--
	self.pageUpButton = Turbine.UI.Button();
	self.pageDownButton = Turbine.UI.Button();

	self.pageUpButton:SetParent(self);
	self.pageUpButton:SetBackColor(Turbine.UI.Color(0,0,0,0));
	self.pageUpButton:SetZOrder(50);
	self.pageUpButton:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
	self.pageUpButton:SetBackColorBlendMode(Turbine.UI.BlendMode.Undefined);


	self.pageDownButton:SetParent(self);
	self.pageDownButton:SetBackColor(Turbine.UI.Color(1,1,1,1));
	self.pageDownButton:SetZOrder(50);
	self.pageDownButton:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
	self.pageDownButton:SetBackColorBlendMode(Turbine.UI.BlendMode.Undefined);

	--[[ Add the toggle direction button ]]--
	self.toggleButton = Turbine.UI.Button();

	self.toggleButton:SetParent(self);
	self.toggleButton:SetBackColor(Turbine.UI.Color(0,0,0,0));
	self.toggleButton:SetZOrder(50);
	self.toggleButton:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
	self.toggleButton:SetBackColorBlendMode(Turbine.UI.BlendMode.Undefined);

	--[[ set values for the quickslot item ]]--
	self.quickslotSize = 36;
	self.quickslots = { };

	--[[ set the location of the quickslot, then initialize 
           to the first item in the list  ]]--
	for i = 1,6,1 do
		self.quickslots[i] = Turbine.UI.Lotro.Quickslot();		
		self.quickslots[i]:SetParent( self );
		self.quickslots[i]:SetZOrder(100);
		self.quickslots[i]:SetSize(self.quickslotSize, self.quickslotSize);
	end

	self:SetControls();
	self:SetQuickslots();

	local travel = self;
	local mainWindow = self;

	------------------------------------------
	--[[ Below are all the event handlers ]]--
	------------------------------------------

	local isMoving = false;
	local x = 0;
	local y = 0;

	self.MouseDown = function(sender, args )
		if (args.Button == Turbine.UI.MouseButton.Left) then
			isMoving = true;
			x = args.X;
			y = args.Y;
		end
	end

	self.MouseUp = function(sender, args )
		if (args.Button == Turbine.UI.MouseButton.Left) then
			isMoving = false;

			local one,two = self:GetPosition();
			self.settings.positionX = one;
			self.settings.positionY = two;
			self:SaveSettings();
		end
	end

	self.MouseMove = function(sender, args )
		if (isMoving) then
			local oldX,oldY = self:GetPosition();
			self:SetPosition(oldX + args.X - x,oldY + args.Y - y);	
		end
	end

	self.toggleButton.Click = function(sender, args)
		if(self.settings.orientation == 1) then
			self.settings.orientation = 2;
		else
			self.settings.orientation = 1;
		end

		self:SetControls();
		self:SetQuickslots();
		self:SaveSettings();
	end

	self.pageUpButton.Click = function(sender, args)
		self.pageNumber = self.pageNumber + 1;

		-- stop it at page 4
		if(self.pageNumber > 4) then
			self.pageNumber = 4;
		end

		self:SetControls();
		self:SetQuickslots();
	end

	self.pageDownButton.Click = function(sender, args)
		self.pageNumber = self.pageNumber - 1;

		-- stop it at page 1
		if(self.pageNumber < 1) then
			self.pageNumber = 1;
		end

		self:SetControls();
		self:SetQuickslots();
	end

	self.MouseEnter = function(sender, args)
		self:SetBackColor(self.bgColor);
	end

	self.MouseLeave = function(sender, args)
		self:SetBackColor(Turbine.UI.Color(0,0,0,0));
	end
end

function pagedSlotsWindow:SetControls()
	self.pageLabel:SetText(string.format("%d",self.pageNumber));

	if(self.settings.orientation == 1) then
		self:SetSize( 250, 46 );

		self.pageLabel:SetSize(20,20)
		self.pageLabel:SetPosition(228,14);
		self.pageLabel:SetVisible( true );

		self.pageUpButton:SetBackground("DhorPlugins/PagedSlots/Resources/UpArrow.tga");
		self.pageUpButton:SetSize(16,16);
		self.pageUpButton:SetPosition(230,0);
		self.pageUpButton:SetVisible( true );

		self.pageDownButton:SetBackground("DhorPlugins/PagedSlots/Resources/DownArrow.tga");
		self.pageDownButton:SetSize(16,16);
		self.pageDownButton:SetPosition(230,30);
		self.pageDownButton:SetVisible( true );

		self.toggleButton:SetBackground("DhorPlugins/PagedSlots/Resources/toggle.tga");
		self.toggleButton:SetSize(14,14);
		self.toggleButton:SetPosition(0,16);
		self.toggleButton:SetVisible( true );
	else
		self:SetSize( 46, 250);

		self.pageLabel:SetSize(20,20)
		self.pageLabel:SetPosition(13,228);
		self.pageLabel:SetVisible( true );

		self.pageUpButton:SetBackground("DhorPlugins/PagedSlots/Resources/RightArrow.tga");
		self.pageUpButton:SetSize(16,16);
		self.pageUpButton:SetPosition(30,230);
		self.pageUpButton:SetVisible( true );

		self.pageDownButton:SetBackground("DhorPlugins/PagedSlots/Resources/LeftArrow.tga");
		self.pageDownButton:SetSize(16,16);
		self.pageDownButton:SetPosition(0,230);
		self.pageDownButton:SetVisible( true );

		self.toggleButton:SetBackground("DhorPlugins/PagedSlots/Resources/toggle.tga");
		self.toggleButton:SetSize(14,14);
		self.toggleButton:SetPosition(16,0);
		self.toggleButton:SetVisible( true );


	end
end

function pagedSlotsWindow:SetQuickslots()

	for i = 1, 6, 1 do
		-- set the properties of the quickslot used to hold the travel shortcut
	
		if(self.settings.orientation == 1) then
			self.quickslots[i]:SetPosition(13 + (i - 1) * self.quickslotSize,5);
		else
			self.quickslots[i]:SetPosition(5,13 + (i - 1) * self.quickslotSize);
		end

		local type = self.settings.shortcuts[i + (self.pageNumber - 1) * 6].Type;
		local data = self.settings.shortcuts[i + (self.pageNumber - 1) * 6].Data;

      myShortcut = Turbine.UI.Lotro.Shortcut(type,data);
      
		self.quickslots[i]:SetShortcut(myShortcut);
		self.quickslots[i]:SetAllowDrop(true);
		self.quickslots[i]:SetZOrder(100);		

		--[[ register the event to handle adding a new shortcut ]]--
		self.quickslots[i].ShortcutChanged = function(sender,args)
		   -- update and save the settings
         self:ChangeShortcut(i);
            			
         -- check if we are going to perform a swap
			if(self.swappingShortcuts and not self.deleting) then
			   
			   -- clear the swap flag because this event will fire when we set
			   -- old slot, and we don't want to do a double swap
	         self.swappingShortcuts = false;
            
            -- set the old quickslot with the shortcut we are replacing
            if(self.draggedIndex ~= 0) then		
				  self.quickslots[self.draggedIndex]:SetShortcut(self.ReplacedShortcut);
            end				
				
			elseif(not self.deleting) then
            -- we are not swapping, but also not deleting, so clear the flags
		      self.draggedIndex = 0;
		      self.swappingShortcuts = false;
			end
         
         -- clear the deleting flag no matter what
         self.deleting = false;
		end
      		
		self.quickslots[i].MouseDown = function(sender,args)
		   -- mouse down means we might be trying a drag, set some variables
			self.draggingShortcut = true;
			self.draggedIndex = i;
			
			-- for now, make sure we are not trying to swap until we know for sure
			self.swappingShortcuts = false;
		end

		self.quickslots[i].MouseLeave = function(sender,args)
		   -- if the mouse button is down, and we leave the quickslot, then we
		   -- are dragging the shortcut out
			if(self.draggingShortcut) then
				self.draggedOut = true;
			end
		end

		self.quickslots[i].MouseUp = function(sender,args)
			if(args.Button == Turbine.UI.MouseButton.Left) then
				if(self.draggedOut) then
				   -- let the ShortcutChanged event know this is a delete event
				   -- then remove the shortcut
				   self.deleting = true;
					self:RemoveShortcut(i);
				end				
			end
			
			-- reset the tracking variables
			self.draggedOut=false;
			self.draggingShortcut = false;
		end
		
		self.quickslots[i].DragEnter = function(sender,args)

         -- get the shortcut of the quickslot we just entered
		   self.ReplacedShortcut = self.quickslots[i]:GetShortcut();
		   
         -- if the shortcut is a real shortcut, then we need to do a swap 
		   if(self.ReplacedShortcut:GetData() ~= "") then
            self.swappingShortcuts = true;
		   else
		      self.swappingShortcuts = false;
		   end
		end
		
		self.quickslots[i].DragDrop = function(sender,args)
		   --Turbine.Shell.WriteLine("I'm a quickslot drop event");
		end
	end
end

function pagedSlotsWindow:RemoveShortcut(i)
	local data = "";
	local type = 0;
		
	self.settings.shortcuts[i + (self.pageNumber - 1) * 6] = { Data=data,Type=type };
	
	self:SaveSettings();
	self:SetQuickslots();
end

function pagedSlotsWindow:ChangeShortcut(i)

	local shortcut = self.quickslots[i]:GetShortcut();
	local data = shortcut:GetData();
	local type = shortcut:GetType(); 
	
	self.settings.shortcuts[i + (self.pageNumber - 1) * 6] = { Data=data,Type=type };
	
	self:SaveSettings();
	self:SetQuickslots();
end


function pagedSlotsWindow:LoadSettings()
	-- load the settings.  If a value is not available, set a default value
	self.loading = true;

	self.settings = Turbine.PluginData.Load( Turbine.DataScope.Character, "pagedSlotsWindowSettings" );

	if ( type( self.settings ) ~= "table" ) then
		self.settings = { };
	end

	if ( not self.settings.positionX ) then
		self.settings.positionX = Turbine.UI.Display.GetWidth() - self:GetWidth() - 250; 
	end

	if ( not self.settings.positionY ) then
		self.settings.positionY = Turbine.UI.Display.GetHeight() - self:GetHeight() - 150 * 1.5;
	end

	if ( not self.settings.orientation ) then
		Turbine.Shell.WriteLine("Why am I here?");
		self.settings.orientation = 1;
	end

	if ( not self.settings.shortcuts ) then
		self.settings.shortcuts = { };		
		for i = 1, 24, 1 do
			local shortcutData = { Type=0,Data="" };
			table.insert(self.settings.shortcuts,shortcutData);					
		end
	end

	self.loading = false;
end

function pagedSlotsWindow:SaveSettings()
	if (self.loading) then
		return;
	end

	-- save the settings
	Turbine.PluginData.Save( Turbine.DataScope.Character, "pagedSlotsWindowSettings", self.settings );
end

function pagedSlotsWindow:ChangeVisible(value)
	self.quickslots[1]:SetVisible(value);
end

function pagedSlotsWindow:Closing(sender,args)
   for i = 1,6,1 do
      self.quickslots[i]:SetShortcut(nil);
   end
end



